Imports System.Math
Imports System.Threading

Public Class Form1
    Inherits System.Windows.Forms.Form

    Private m_Parallel As Boolean = False

    Private m_DrawingBox As Boolean
    Private m_StartX As Double
    Private m_StartY As Double
    Private m_CurX As Double
    Private m_CurY As Double

    Private m_Xmin As Double
    Private m_Xmax As Double
    Private m_Ymin As Double
    Private m_Ymax As Double

    Public MaxIterations As Integer
    Public Zr As Double
    Public Zim As Double
    Public Z2r As Double
    Public Z2im As Double

    Public NumColors As Integer
    Private m_Colors() As Color

    Private m_Bm As Bitmap
    Private m_BmColors(,) As Integer

    Private Const MIN_X As Double = -2.2
    Private Const MAX_X As Double = 1
    Private Const MIN_Y As Double = -1.2
    Private Const MAX_Y As Double = 1.2

    ' Return this color's value.
    ReadOnly Property Color(ByVal Index As Integer) As Color
        Get
            Color = m_Colors(Index)
        End Get
    End Property

    ' Reset the number of colors to 0.
    Public Sub ResetColors()
        NumColors = 0
        Erase m_Colors
    End Sub

    ' Add this color to the list.
    Public Sub AddColor(ByVal new_color As Color)
        NumColors += 1
        ReDim Preserve m_Colors(NumColors - 1)
        m_Colors(NumColors - 1) = new_color
    End Sub

    ' Adjust the aspect ratio of the selected
    ' coordinates so they fit the window properly.
    Private Sub AdjustAspect()
        Dim want_aspect As Double
        Dim picCanvas_aspect As Double
        Dim hgt As Double
        Dim wid As Double
        Dim mid As Double

        want_aspect = (m_Ymax - m_Ymin) / (m_Xmax - m_Xmin)
        picCanvas_aspect = picCanvas.ClientSize.Height / picCanvas.ClientSize.Width
        If want_aspect > picCanvas_aspect Then
            ' The selected area is too tall and thin.
            ' Make it wider.
            wid = (m_Ymax - m_Ymin) / picCanvas_aspect
            mid = (m_Xmin + m_Xmax) / 2
            m_Xmin = mid - wid / 2
            m_Xmax = mid + wid / 2
        Else
            ' The selected area is too short and wide.
            ' Make it taller.
            hgt = (m_Xmax - m_Xmin) * picCanvas_aspect
            mid = (m_Ymin + m_Ymax) / 2
            m_Ymin = mid - hgt / 2
            m_Ymax = mid + hgt / 2
        End If
    End Sub

    Dim wid As Integer
    Dim hgt As Integer
    Dim dReaC As Double
    Dim dImaC As Double

    ' Draw the Mandelbrot set.
    Private Sub DrawMandelbrot()
        ' Make a Bitmap to draw on.
        m_Bm = New Bitmap(picCanvas.ClientSize.Width, picCanvas.ClientSize.Height)
        Dim gr As Graphics = Graphics.FromImage(m_Bm)
        ReDim m_BmColors(picCanvas.ClientSize.Width - 1, picCanvas.ClientSize.Height - 1)

        ' Clear.
        gr.Clear(picCanvas.BackColor)
        picCanvas.Image = m_Bm
        Application.DoEvents()

        ' Adjust the coordinate bounds to fit picCanvas.
        AdjustAspect()

        ' dReaC is the change in the real part
        ' (X value) for C. dImaC is the change in the
        ' imaginary part (Y value).
        wid = picCanvas.ClientRectangle.Width
        hgt = picCanvas.ClientRectangle.Height
        dReaC = (m_Xmax - m_Xmin) / (wid - 1)
        dImaC = (m_Ymax - m_Ymin) / (hgt - 1)

        ' Calculate the values.
        Dim start_time As Date = Now
        If m_Parallel Then
            Parallel.For(0, wid, AddressOf DrawXLine)
        Else
            For x As Integer = 0 To wid - 1
                DrawXLine(x)
            Next x
        End If
        Dim stop_time As Date = Now

        ' Copy the values into the bitmap.
        For x As Integer = 0 To wid - 1
            For y As Integer = 0 To hgt - 1
                m_Bm.SetPixel(x, y, m_Colors(m_BmColors(x, y) Mod NumColors))
            Next y
        Next x
        picCanvas.Invalidate()

        Dim elapsed_time As TimeSpan = stop_time - start_time
        Dim ch As String = If(m_Parallel, "P", "S")
        Text = "Mandelbrot " & ch & " (" & _
            m_Xmin.ToString("0.000000") & ", " & _
            m_Ymin.ToString("0.000000") & ")-(" & _
            m_Xmax.ToString("0.000000") & ", " & _
            m_Ymax.ToString("0.000000") & ") -- " & _
            elapsed_time.TotalSeconds.ToString("0.00") & " secs"
    End Sub

    Private Sub DrawXLine(ByVal X As Integer)
        ' Work until the magnitude squared > 4.
        Const MAX_MAG_SQUARED As Integer = 4

        Dim ReaC As Double = m_Xmin + X * dReaC
        Dim ImaC As Double = m_Ymin
        For Y = 0 To hgt - 1
            Dim ReaZ As Double = Zr
            Dim ImaZ As Double = Zim
            Dim ReaZ2 As Double = Z2r
            Dim ImaZ2 As Double = Z2im
            Dim clr As Integer = 1
            Do While clr < MaxIterations And ReaZ2 + ImaZ2 < MAX_MAG_SQUARED
                ' Calculate Z(clr).
                ReaZ2 = ReaZ * ReaZ
                ImaZ2 = ImaZ * ImaZ
                ImaZ = 2 * ImaZ * ReaZ + ImaC
                ReaZ = ReaZ2 - ImaZ2 + ReaC
                clr = clr + 1
            Loop

            ' Set the pixel's value.

            m_BmColors(X, Y) = clr

            ImaC = ImaC + dImaC
        Next Y
        ReaC = ReaC + dReaC
    End Sub

    ' Scale the selected area by this factor.
    Private Sub ScaleArea(ByVal scale_factor As Integer)
        Dim size As Double
        Dim mid As Double

        size = scale_factor * (m_Xmax - m_Xmin)
        If size > 3.2 Then
            mnuScaleFull_Click(Nothing, Nothing)
            Exit Sub
        End If
        mid = (m_Xmin + m_Xmax) / 2
        m_Xmin = mid - size / 2
        m_Xmax = mid + size / 2

        size = scale_factor * (m_Ymax - m_Ymin)
        If size > 2.4 Then
            mnuScaleFull_Click(Nothing, Nothing)
            Exit Sub
        End If
        mid = (m_Ymin + m_Ymax) / 2
        m_Ymin = mid - size / 2
        m_Ymax = mid + size / 2

        Me.Cursor = Cursors.WaitCursor
        Application.DoEvents()
        DrawMandelbrot()
        Me.Cursor = Cursors.Default
        picCanvas.Cursor = Cursors.Cross
    End Sub

    Private Sub mnuScale_2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuScale_2.Click
        ScaleArea(2)
    End Sub
    Private Sub mnuScale_4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuScale_4.Click
        ScaleArea(4)
    End Sub
    Private Sub mnuScale_8_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuScale_8.Click
        ScaleArea(8)
    End Sub
    ' Zoom out to full scale.
    Private Sub mnuScaleFull_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuScaleFull.Click
        m_Xmin = MIN_X
        m_Xmax = MAX_X
        m_Ymin = MIN_Y
        m_Ymax = MAX_Y

        Me.Cursor = Cursors.WaitCursor
        Application.DoEvents()
        DrawMandelbrot()
        Me.Cursor = Cursors.Default
        picCanvas.Cursor = Cursors.Cross
    End Sub

    ' Select parallel or single processing.
    Private Sub Form1_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles Me.KeyPress
        ' Determine whether we're running in parallel or single.
        m_Parallel = (Char.ToLower(e.KeyChar) = "p"c)

        ' Redraw the current area.
        ScaleArea(1)
    End Sub

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Me.Show()
        Application.DoEvents()

        MaxIterations = 64

        ' Create some default colors.
        ResetColors()
        Dim frm As New frmConfig
        AddColor(frm.picColor_40.BackColor)
        AddColor(frm.picColor_17.BackColor)
        AddColor(frm.picColor_18.BackColor)
        AddColor(frm.picColor_19.BackColor)
        AddColor(frm.picColor_20.BackColor)
        AddColor(frm.picColor_21.BackColor)
        AddColor(frm.picColor_22.BackColor)
        AddColor(frm.picColor_23.BackColor)
        frm.Close()

        ' Display the first Mandelbrot set.
        mnuScaleFull_Click(Nothing, Nothing)
    End Sub

    Private Sub mnuOptOptions_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuOptOptions.Click
        Dim frm As New frmConfig
        frm.Initialize(Me)
        frm.ShowDialog()
    End Sub

    ' Refresh. This is useful when the user resizes the form.
    Private Sub mnuScaleRefresh_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuScaleRefresh.Click
        ScaleArea(1)
    End Sub

    Private Sub picCanvas_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseDown
        m_DrawingBox = True
        m_StartX = e.X
        m_StartY = e.Y
        m_CurX = e.X
        m_CurY = e.Y
    End Sub

    Private Sub picCanvas_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseMove
        If Not m_DrawingBox Then Exit Sub

        m_CurX = e.X
        m_CurY = e.Y

        Dim bm As New Bitmap(m_Bm)
        Dim gr As Graphics = Graphics.FromImage(bm)
        gr.DrawRectangle(Pens.Yellow, _
            CInt(Min(m_StartX, m_CurX)), CInt(Min(m_StartY, m_CurY)), _
            CInt(Abs(m_StartX - m_CurX)), CInt(Abs(m_StartY - m_CurY)))
        picCanvas.Image = bm
    End Sub

    Private Sub picCanvas_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseUp
        If Not m_DrawingBox Then Exit Sub
        m_DrawingBox = False
        picCanvas.Image = m_Bm

        m_CurX = e.X
        m_CurY = e.Y

        Dim x1 As Double
        Dim x2 As Double
        Dim y1 As Double
        Dim y2 As Double
        Dim factor As Double

        ' Put the coordinates in proper order.
        x1 = Min(m_StartX, m_CurX)
        x2 = Max(m_StartX, m_CurX)
        If x1 = x2 Then x2 = x1 + 1

        y1 = Min(m_StartY, m_CurY)
        y2 = Max(m_StartY, m_CurY)
        If y1 = y2 Then y2 = y1 + 1

        ' Convert screen coords into drawing coords.
        factor = (m_Xmax - m_Xmin) / picCanvas.ClientSize.Width
        m_Xmax = m_Xmin + x2 * factor
        m_Xmin = m_Xmin + x1 * factor

        factor = (m_Ymax - m_Ymin) / picCanvas.ClientSize.Height
        m_Ymax = m_Ymin + y2 * factor
        m_Ymin = m_Ymin + y1 * factor

        Me.Cursor = Cursors.WaitCursor
        Application.DoEvents()
        DrawMandelbrot()
        Me.Cursor = Cursors.Default
        picCanvas.Cursor = Cursors.Cross
    End Sub
End Class
